#!/usr/bin/env python
import sys
import os
import gem
import time
gem.loglevel("debug")

## global configuration
# number of threads to use for each run
THREADS = 8
#QUALITY = None
# remove intermediate files
REMOVE_FILES = False

## make sure we have access to the demo data
#base_dir = os.path.split(os.path.dirname(os.path.abspath(__file__)))[0]
base_dir = "/users/rg/fcamara/Research/geneid_training/P.cucumerina/data/set1"
result_dir = "%s/results_Pc_new" % base_dir
data_dir = "%s" % base_dir
index = "%s/Pcucumerina.gem"  % result_dir

if not os.path.exists(base_dir):
    print("Unable to find the demo data."
          "Please make sure you ran the bootstrap "
          "script to download and prepare demo data")
    exit(1)


def create_index():
    """Create the example index"""
    #indicate file name of the genome the reads are to be mapped to
    print "Preparing index"
    return gem.index("%s/Pcucumerina.fa" % data_dir, "%s/Pcucumerina.gem" % result_dir, threads=THREADS)


if __name__ == "__main__":
    ## define the input
    #
    # the gem index
    index = create_index()
    # the input reads (multiplexed fastq)
    reads = "%s/Pc-all.fastq" % base_dir
    # the input reads in case of paired-end reads
    # input_file = "%s/spleen_1_cl.fastq" % base_dir
    # input_file2 = "%s/spleen_2_cl.fastq" % base_dir

   # print reads;
    # the annotation used for junction site detection (in case there is one)
   # annotation = "%s/chr21.gtf" % data_dir

    # define some output file name
    #
    # global name for the resulting data-set
    name = "%s/Pcucumerina_mapping" % result_dir
    # initial mapping output
    initial_out = "%s_initial.map" % name
    # output from the denovo junction detection step
    #denovo_out = "%s_denovo.map" % name
    # denovo junction sites
    junctions_out = "%s.junctions" % name
    # initial splitmap output
    initial_split_out = "%s_initial_split.map" % name
    transcriptome_out = "%s.transcriptome.map" % name
    index_denovo_out = "%s.idx.transcriptome.gem" % name
    # second round mapping with 3" end trimmed by 20 bases
   # trim_20_out = "%s_trim_20.map" % name
    # second round split mapping with 3" end trimmed by 20 bases
   # trim_20_split_out = "%s_trim_20_split.map" % name
    # first end second mapping step merged
    merge_out = "%s_merged.map" % name
    # paired mappings
#    paired_out = "%s_paired.map" % name
    # scored mappings
#    final_out = "%s.map" % name
    # sam/bam output
    sam_out = "%s.bam" % name

    ## Create initial mapping
    # we deal with a single file with interleaved paired reads here, but
    # creating input from two files for the read pairs is straight forward using the interleave filter
    #
   # input_1 = gem.files.open(input_file)
   # input_2 = gem.files.open(input_file2)
   # input = gem.filter.interleave([input_1, input_2])
    print "Running initial mapping"
    
    input = gem.files.open(reads)
    
    if not os.path.exists(initial_out):
       initial_mapping = gem.mapper(input, index, initial_out, mismatches=0.04, quality=64, max_edit_distance=0, delta=0, threads=THREADS)
    else:
        initial_mapping = gem.files.open(initial_out)
    
   # 
  #  initial_mapping_file = "%s/Pc-all_mapping_initial.map" % result_dir
  #  initial_mapping = gem.files.open(initial_mapping_file)

  #  print initial_mapping
    ## junction sites
    # before we can do the split mapping, we have to load
    # the junction sites from a gtf annotation and
    # run the denovo-junction detection. This will also give
    # us a mapping that preserves short indels detected during the
    # extraction run.
  #  print "Loading GTF junctions from %s" % annotation
  #  junctions = gem.junctions.from_gtf(annotation)

    # now the denovo run. This returns a tuple : (mapping, junctions)
    # and here we use the merge_with parameter to merge the denovo junctions with
    # the previously loaded gtf junctions.
    #
    # Also note that we pass only unmapped reads from the initial mapping to the junction
    # extraction. This is done using the "unmapped filter"
    print "Getting de-novo junctions"
    #junctions = ""

    if not os.path.exists(junctions_out):
    
        junctions = gem.extract_junctions(
        gem.filter.unmapped(initial_mapping), # only unmapped reads from the initial mapping
        index,
       # denovo_out,
        quality=64,
        mismatches=0.04,
        threads=THREADS
        #merge_with=set(junctions)
        )
 
    ## we filter the junnctions now by their distance and
    # write all junctions with a distance <= 500000 to a file
        print "Writing junctions file"
        gem.junctions.write_junctions(gem.junctions.filter_by_distance(junctions, 20, 20000), junctions_out, index)



    ## Initial split map run with junction sites
    # Here we take all unmapped reads after the denovo junction detection and
    # pass them to the split mapper along with the junctions
  #  print "Running initial split-map"
   # initial_split_mapping = gem.splitmapper(
   #     gem.filter.unmapped(denovo_mapping),
   #     index,
   #     initial_split_out,
   #     quality=64,
   #     junctions_file=junctions_out,
   #     mismatches=0.06,
   #     threads=THREADS)


    if not os.path.exists(transcriptome_out):    
        print "Computing denovo transcriptome"
        (denovo_transcriptome, denovo_keys) = gem.compute_transcriptome(150, index, junctions_out)
        print "Indexing denovo transcriptome"
       # timer.stop("Transcriptome generated in %s")
       # timer = Timer()
   # index_denovo_out = "%s.idx.transcriptome.gem" % name
        idx = gem.index(denovo_transcriptome, index_denovo_out, threads=THREADS)
       # timer.stop("Transcriptome indexed in %s")
        
        indices = [idx]
        keys = [denovo_keys]
   # transcriptome_out = "%s.transcriptome.map" % name
        transcriptome_mapping = gem.transcript_mapper(
                                initial_mapping,
                                indices,
                                keys,
                                transcriptome_out,
                                mismatches=0.06,
                                quality=64,
                                min_decoded_strata=0,
                                delta=0,
                                threads=THREADS
                                )
    else:
        transcriptome_mapping = gem.files.open(transcriptome_out)
    ## Phase two, hard trim 20 bases from the right side
    # and try mapping again
   # print "Running trim 20 mapping"
   # trim_20_mapping = gem.mapper(
    #    gem.filter.unmapped(initial_split_mapping),
    #    index,
    #    trim_20_out,
    #    trim=(0, 20),  # this is the additional parameter to control the trimming
    #    threads=THREADS)

    ## second round split mapping
   # print "Running trim 20 split-map"
   # trim_20_split_mapping = gem.splitmapper(
  #      gem.filter.unmapped(trim_20_mapping),
   #     index,
   #     trim_20_split_out,
   #     trim=(0, 20),  # this is the additional parameter to control the trimming
   #     junctions_file=junctions_out,
   #     threads=THREADS)


    ## We are done with the basic mapping pipeline,
    # now we merge everything into the final file
    # Note that we call the .clone() method on most
    # of the objects we pass to the merger. This is because the
    # mapping steps return iterators over the results and
    # we eventually iterated through them already once. The
    # .clone() makes sure we new iterator that starts at the beginning
    if not os.path.exists(merge_out):
        print "Merging results"
        merged = gem.merger(
        initial_mapping.clone(), ## this is the main target and must contain ALL reads
        [transcriptome_mapping.clone()]
   #      trim_20_mapping.clone(),
    #     trim_20_split_mapping]
        ).merge(merge_out) # write to file


    ## remove files
    if REMOVE_FILES:
        print "Removing intermediate files"
       # os.remove(initial_out)
       # os.remove(initial_split_out)
       # os.remove(denovo_out)
       # os.remove(junctions_out)
       # os.remove(trim_20_out)
       # os.remove(trim_20_split_out)

    ## pair align the mappings #insert size large to allow for chimeras etc..
   # if not os.path.exists(paired_out):
    #    print "Running pair aligner"
    #    paired_mapping = gem.pairalign(merged, index, paired_out, quality=64, max_insert_size=100000, threads=THREADS)
   # else:
    #    paired_mapping = gem.files.open(paired_out)

  #  if REMOVE_FILES:
  #      os.remove(merge_out)

    ## validate and score the alignment
  #  print "Validating and scoring alignment"
  # scored = gem.score(paired_mapping, index, final_out, threads=THREADS)
  #  scored = gem.score(merged, index, final_out, threads=THREADS)
  #  if REMOVE_FILES:
  #      os.remove(paired_out)

    ## convert the result to sam and then to bam
    print "Converting to sam"
    sam = gem.gem2sam(merged, index, quality=64, threads=THREADS)
    bam = gem.sam2bam(sam, sam_out, sorted=True, sort_memory="1000000000")

    print "Done :)"
